// Shelf support bracket
//  for Whirlpool freezer
// Ed Nisley KE4ZNU October 2012

// Layout options

Layout = "Build";
                    // Overall layout: Show
                    // Printing plates: Build
                    // Parts: Post Base Keystone Support

//- Extrusion parameters must match reality!
//  Print with +1 shells and 3 solid layers

ThreadThick = 0.25;
ThreadWidth = 2.0 * ThreadThick;

function IntegerMultiple(Size,Unit) = Unit * ceil(Size / Unit);

Protrusion = 0.1;           // make holes end cleanly

//----------------------
// Dimensions

PostLength = 17.5;
PostWidth = 8.2;
PostHeight = 14.4;
PostOffset = 4.4;

PostTopWidth = 4.0;
PostTopHeight = 4.2;

BaseLength = 22.6;
BaseWidth = 20.8;
BaseThick = 5.0;

KeystoneOffset = 3.4;
KeyThick = IntegerMultiple(3.0,ThreadThick);
KeyBase = 2.5;
SlotOpening = 11.63;


//----------------------
// Useful routines

module ShowPegGrid(Space = 10.0,Size = 1.0) {

    Range = floor(50 / Space);

    for (x=[-Range:Range])
        for (y=[-Range:Range])
            translate([x*Space,y*Space,Size/2])
            %cube(Size,center=true);

}

//-------------------
// Component parts

//--- Post

module Post(h=PostLength) {

PostTopAngle = atan((PostWidth - PostTopWidth)/(2*PostTopHeight));
PostBottomRadius = PostWidth/2;

PostPolyTop = [PostTopWidth/2,0];
PostPolyBottom = [PostWidth/2,-PostTopHeight];

    hull()
        linear_extrude(height=h) {
            polygon(points=[
                [-PostPolyTop[0],PostPolyTop[1]],
                PostPolyTop,
                PostPolyBottom,
                [-PostPolyBottom[0],PostPolyBottom[1]]
            ]);
            translate([0,-PostHeight + PostBottomRadius])
                circle(r=PostBottomRadius,$fn=4*8);
        }
}

//--- Base block

module Base() {

    linear_extrude(height=BaseThick)
        square([BaseWidth,BaseLength],center=true);

}

//-- Keystone slot

module Keystone() {

Tx = SlotOpening/2 + KeyBase;

    rotate([90,0,0])
        linear_extrude(height=BaseLength)
            polygon(points=[
                [-Tx,KeyThick],
                [ Tx,KeyThick],
                [ SlotOpening/2,0],
                [ SlotOpening/2,-Protrusion],
                [-SlotOpening/2,-Protrusion],
                [-SlotOpening/2,0]
            ]);
}

//--- Support structure

module Support() {

SupportLength = BaseLength - 2*ThreadWidth;
SupportWidth = 2*ThreadWidth;
SupportHeight = KeyThick - Protrusion;

SupportPeriod = 7.0*ThreadWidth;

SupportBeams = 3;           // must be odd -- choose to fit
SIndex = floor((SupportBeams - 1)/2);

    for (i=[-SIndex:SIndex])
        translate([(i*SupportPeriod - SupportWidth/2),-(SupportLength + ThreadWidth),0])
            color("Yellow") cube([SupportWidth,SupportLength,SupportHeight]);
}

//--- The whole thing!

module Bracket(ShowSupp) {

    union() {
        difference() {
            Base();
            translate([0,(BaseLength/2 - KeystoneOffset),0])
                Keystone();
        }
        translate([0,(BaseLength/2 - PostOffset),BaseThick - Protrusion])
            Post(h=(PostLength + Protrusion));
    }

    if (ShowSupp)
        translate([0,(BaseLength/2 - KeystoneOffset),0])
            Support();

}

//----------------------
// Build it!

ShowPegGrid();

if (Layout == "Show")
    Bracket(false);

if (Layout == "Build")
    Bracket(true);

if (Layout == "Post")
    Post();

if (Layout == "Base")
    Base();

if (Layout == "Keystone")
    Keystone();

if (Layout == "Support") {
    Support();
//   Keystone();
}